 use "io"
  use "strbuf"
 use "dict"
  use "error"
 
const WHITE=" \n\t\r" 
  const HTML_DOCTYPE=0
 const HTML_START_TAG=1
 const HTML_CLOSE_TAG=2
 const HTML_SELF_CLOSE=3
 const HTML_TEXT=4
 const HTML_COMMENT=5
 const HTML_EOF=6
 
 type htmlToken {
  typ:Int }
 def htmlToken.new(t:Int) { if (t > 6 || t < 0) error(ERR_ILL_ARG, "The html element type for "+t+" does not exist")
   this.typ=t }
 def htmlToken.get_typ():Int=this.typ
 
 type htmlComment < htmlToken { info:String }
 def htmlComment.new(inf:String) {
  super(HTML_COMMENT)
  this.info=inf }
 def htmlComment.get_info():String = this.info
  def htmlComment.tostr():String = "<!-- "+this.info+" -->"
 
 type htmlText < htmlToken { text:String }
  def htmlText.new(txt:String) {
    super(HTML_TEXT)
    this.text=txt  }
 def htmlText.get_text():String = this.text
  def htmlText.tostr():String=this.text
 
 type htmlCloseTag < htmlToken {
   name:String }
 def htmlCloseTag.new(nam:String) { super(HTML_CLOSE_TAG) this.name=nam }
  def htmlCloseTag.get_name():String = this.name
  def htmlCloseTag.tostr():String="</"+this.name+">"
 
 type htmlDocType < htmlToken { decl:String }
 def htmlDocType.new(dec:String) { super(HTML_DOCTYPE) this.decl=dec }
 def htmlDocType.get_decl():String = this.decl
  def htmlDocType.tostr():String = "<!DOCTYPE "+this.decl+">"
 
 type htmlStartTag < htmlToken {
  name:String,
  attribs:Dict }
 
 def htmlStartTag.new(nam:String, attr:Dict=null) {
   super(HTML_START_TAG)
   this.name=nam
   this.attribs=if (attr == null) new Dict() else attr }
 
 def htmlStartTag.get_name():String = this.name
 def htmlStartTag.get(key:String):String = this.attribs[key].cast(String) 
  def htmlStartTag.set(key:String,value:String) { this.attribs[key]=value } 
 def htmlStartTag.remove(key:String) = this.attribs.remove(key)
 def htmlStartTag.clear()=this.attribs.clear()
 def htmlStartTag.size():Int=this.attribs.size()
  def htmlStartTag.tostr():String {
  var buf:StrBuf=new StrBuf().append("<").append(this.name)
   var keys=this.attribs.keys()
   var i:Int=0
   while(i < keys.len) { buf.append(' ').append(keys[i]).append("=\"").append(this.attribs[keys[i]]).append("\"") i+=1 }
   buf.append(">").tostr()  }
 
 
  type htmlSelfClose < htmlToken {
  name:String,
  attribs:Dict }
 
 def htmlSelfClose.new(nam:String, attr:Dict=null) {
   super(HTML_SELF_CLOSE)
   this.name=nam
   this.attribs=if (attr == null) new Dict() else attr }
 
  def htmlSelfClose.get_name():String = this.name
 def htmlSelfClose.get(key:String):String = this.attribs[key].cast(String) 
  def htmlSelfClose.set(key:String,value:String) { this.attribs[key]=value }
 
 def htmlSelfClose.remove(key:String)=this.attribs.remove(key)
 def htmlSelfClose.clear()=this.attribs.clear()
 def htmlSelfClose.size():Int=this.attribs.size()
 
  def htmlSelfClose.tostr():String {
  var buf:StrBuf=new StrBuf().append("<").append(this.name)
   var keys=this.attribs.keys()
   var i:Int=0
   while(i < keys.len) { buf.append(' ').append(keys[i]).append("=\"").append(this.attribs[keys[i]]).append("\"") i+=1 }
   buf.append("/>").tostr()  }


 type htmlEOF < htmlToken { }
 def htmlEOF.new() { super(HTML_EOF) }
 
 
 def htmlToken.tostr():String { var str:String switch(this.typ) {
 HTML_DOCTYPE: str= this.cast(htmlDocType).tostr()
 HTML_COMMENT: str=this.cast(htmlComment).tostr()
  HTML_START_TAG: str=this.cast(htmlStartTag).tostr()
  HTML_CLOSE_TAG: str= this.cast(htmlCloseTag).tostr()
  HTML_SELF_CLOSE: str= this.cast(htmlSelfClose).tostr()
  HTML_TEXT: str= this.cast(htmlText).tostr()
 HTML_EOF: error(ERR_ILL_STATE, "Html EOF cannot be converted to String")
 } str }
 
 
// eof
